use std::io::{self, Cursor};

use ciborium::into_writer;
use itertools::izip;
use serde::{Deserialize, Serialize};
use time::PrimitiveDateTime;

use crate::json::{self};

use super::{area::Area, image::Image, link::Link, text::Text};

pub type Cbor = Vec<u8>;

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Alert {
    pub id: String,
    title: Text,
    description: Text,
    nation_wide: bool,
    instructions: Vec<Text>,
    sender: String,
    publisher: Option<String>,
    links: Vec<Link>,
    contact: Text,
    event: String,
    all_clear: bool,
    severity: String,
    publish_date: PrimitiveDateTime,
    areas: Vec<Area>,
    images: Vec<Image>,
}

impl From<json::alert::Alert> for Alert {
    fn from(value: json::alert::Alert) -> Self {
        Self {
            id: value.identifier.clone(),
            title: value.title.into(),
            description: value.description.into(),
            nation_wide: value.nation_wide,
            instructions: value.instructions.into_iter().map(|x| x.into()).collect(),
            sender: value.sender,
            publisher: value.publisher_name,
            links: value.links.into_iter().map(|x| x.into()).collect(),
            contact: value.contact.into(),
            event: value.event,
            all_clear: value.all_clear,
            severity: value.severity,
            publish_date: value.publish_date,
            areas: value.areas.into_iter().map(|x| x.into()).collect(),
            images: izip!(
                &value.images,
                &value.image_titles,
                &value.image_descriptions
            )
            .map(|(img, title, desc)| Image {
                uri: img.uri.clone(),
                digest: img.digest.clone(),
                title: title.title.clone().into(),
                alt_text: title.alt_text.clone(),
                description: desc.description.clone().into(),
            })
            .collect(),
        }
    }
}

impl TryFrom<Alert> for Cbor {
    type Error = ciborium::ser::Error<io::Error>;

    fn try_from(value: Alert) -> Result<Self, Self::Error> {
        let mut buffer = Vec::new();
        let writer = Cursor::new(&mut buffer);
        into_writer(&value, writer)?;

        Ok(buffer)
    }
}
