use std::time::Duration;

use config::Config;
use data::alert::Cbor;
use datastore::rdbms::Db;
use error::Error;
use futures::{stream, StreamExt};
use tokio::time;
use tracing::{error, info};

use crate::json::alert::Alert;

pub mod alert_swiss;
pub mod cli;
pub mod config;
pub mod data {
    pub mod alert;
    pub mod area;
    pub mod circle;
    pub mod coordinate;
    pub mod image;
    pub mod link;
    pub mod polygon;
    pub mod text;
}
pub mod error;
pub mod datastore {
    pub mod rdbms;
}
mod json {
    pub mod alert;
    pub mod alerts;
    pub mod area;
    pub mod circle;
    pub mod contact;
    pub mod coordinate;
    pub mod datetime;
    pub mod description;
    pub mod image;
    pub mod image_description;
    pub mod image_title;
    pub mod link;
    pub mod polygon;
    pub mod region;
    pub mod text;
    pub mod title;
}
pub mod logging;
pub mod mqtt;

async fn handle_alerts(db: &Db) -> Result<Vec<Alert>, Error> {
    info!("checking alerts");

    let alerts = alert_swiss::fetch_alerts().await?;
    let alerts: Vec<Alert> = alerts
        .list
        .into_iter()
        .filter(|alert| !alert.identifier.starts_with("TEST-"))
        .collect();

    let new_alerts: Vec<Alert> = stream::iter(alerts)
        .filter_map(|alert| async move {
            if db.is_alert_updated(&alert).await.ok().unwrap_or(false) {
                Some(alert)
            } else {
                None
            }
        })
        .collect()
        .await;

    stream::iter(new_alerts.clone())
        .for_each_concurrent(None, |alert| async move {
            info!("saving alert {}", alert.identifier);
            match db.save_alert(&alert).await {
                Ok(_) => {}
                Err(e) => error!("failed to save alert: {e}"),
            };
        })
        .await;
    Ok(new_alerts)
}

pub async fn run(config: Config) -> Result<(), Error> {
    let mut interval = time::interval(Duration::from_secs(config.interval));

    let db = &datastore::rdbms::connect(&config.connection).await?;
    let mqtt = mqtt::connect(&config.mqtt);

    loop {
        interval.tick().await;

        let new_alerts = handle_alerts(db).await?;
        stream::iter(new_alerts)
            .for_each(|alert| async {
                let alert: data::alert::Alert = alert.into();
                match <Cbor as TryFrom<data::alert::Alert>>::try_from(alert) {
                    Ok(cbor) => {
                        if let Err(e) = mqtt.publish(&cbor).await {
                            error!("failed pusblishing to mqtt: {e:?}")
                        }
                    }
                    Err(e) => error!("failed to serialize to cbor: {e:?}"),
                };
            })
            .await;
    }
}
